@@ -23,8 +23,8 @@
 # ̥饤֥
 #
 # $RCSfile: libcommon.pl,v $
-# $Revision: 1.123 $
-# $Date: 2009/01/05 06:44:41 $
+# $Revision: 1.145 $
+# $Date: 2013/02/08 01:00:56 $
 #
 ##########################################################
 
@@ -33,7 +33,7 @@ use vars qw/%LANG $L_LOG $L_DISP $LOGLANG/;
 use vars qw/$DHCPDDIR $ETCDIR $WORKDIR $WORKDIR $DHCPDCONFTMP $DHCPADMINCONF $LOCKFILE/;
 use vars qw/$DHCPDCONF $TEMPDIR $TABFILE $TITLEFILE $IMAGEDIR $SERVERLISTFILE/;
 use vars qw/$USERLISTFILE/;
-use vars qw/$DISPLANG $LOGLANG $L_DISP $L_LOG/;
+use vars qw/$DISPLANG $LOGLANG $L_DISP $L_LOG $OTHERDIR/;
 
 #############ե롦ǥ쥯ȥ#############
 
@@ -50,12 +50,14 @@ $TITLEFILE = $TEMPDIR . "/title";
 $IMAGEDIR = $DHCPDDIR . "/cgi/image";
 $SERVERLISTFILE = $ETCDIR . "/serverlist";
 $USERLISTFILE = $ETCDIR . "/usertable";
+$OTHERDIR = "_other";
 
 ##################ؿ⥰Х##################
 
 my @USELANG = ("en_US", "ja_JP.eucJP");
 $DISPLANG = "DHCPADMIN_LANG";
-$LOGLANG = "en_US";
+#$LOGLANG = "en_US";
+$LOGLANG = "ja_JP.eucJP";
 $L_DISP = 0;
 $L_LOG = 1;
 my $DIRMODE = 0755;
@@ -498,7 +500,6 @@ sub make_dir
     return 0;
 }
 
-
 #
 # եɤ߹سǼ
 #
@@ -682,7 +683,7 @@ sub check_domain
     }
 
     # ʸå
-    $pattern ='^[A-Za-z0-9\-.]*$';
+    $pattern ='^[A-Za-z0-9\-.,]*$';
     if ($domain !~ /$pattern/) {
         return 1;
     }
@@ -993,7 +994,7 @@ sub syserr
 }
 
 #
-# ǥ쥯ȥ
+# WORLDIR۲Υǥ쥯ȥ
 #
 # []
 #       arg1    ǥ쥯ȥ̾
@@ -1007,26 +1008,54 @@ sub syserr
 sub del_dir
 {
     my ($deldirpath, $errmsg) = @_;
+    my @tmpdirlist;
     my @tmpfilelist;
     my $delfilename;
-
+    my $filepath;
+    my $ret;
+ 
     # ǥ쥯ȥ۲Υե
-    if (read_dir($deldirpath, \@tmpfilelist, $errmsg) < 0 ) {
+    if (read_dir($deldirpath, \@tmpdirlist, $errmsg) < 0 ) {
         return -1;
     }
 
-    # եκ
-    foreach $delfilename (@tmpfilelist) {
-        unlink($delfilename);
+    foreach $filepath (@tmpdirlist) {
+    
+        if(read_dir($filepath, \@tmpfilelist, $errmsg) < 0) {
+            return -1;
+        }
+
+        # եκ
+        foreach $delfilename (@tmpfilelist) {
+            $ret = unlink($delfilename);
+
+            # ˼Ԥ
+            if ($ret == 0) {
+                $$errmsg[$L_DISP] = $LANG{$ENV{$DISPLANG}}{'ERR_TMP_FILE3'}
+                                    . "(" . $delfilename . ")";
+                $$errmsg[$L_LOG] = $LANG{$LOGLANG}{'ERR_TMP_FILE3'}
+                                    . "(" . $delfilename . ")";
+                return -1;
+            }
+        }
+
+        # ǥ쥯ȥκ
+        if (rmdir($filepath) ==  0) {
+            $$errmsg[$L_DISP] = $LANG{$ENV{$DISPLANG}}{'ERR_TMP_DIR2'}
+                           . "(" . $filepath . ")";
+            $$errmsg[$L_LOG] = $LANG{$LOGLANG}{'ERR_TMP_DIR2'}
+                          . "(" . $filepath . ")";
+            return -1;
+        }
     }
 
-    # ǥ쥯ȥκ
+    # Υǥ쥯ȥκ
     if (rmdir($deldirpath) ==  0) {
-        $$errmsg[$L_DISP] = $LANG{$ENV{$DISPLANG}}{'ERR_TMP_DIR2'}
+         $$errmsg[$L_DISP] = $LANG{$ENV{$DISPLANG}}{'ERR_TMP_DIR2'}
                             . "(" . $deldirpath . ")";
-        $$errmsg[$L_LOG] = $LANG{$LOGLANG}{'ERR_DHCPD_CONF_FORM'}
-                           . "(" . $deldirpath . ")";
-        return -1;
+         $$errmsg[$L_LOG] = $LANG{$LOGLANG}{'ERR_TMP_DIR2'}
+                            . "(" . $deldirpath . ")";
+         return -1;
     }
 
     return 0;
@@ -1037,6 +1066,7 @@ sub del_dir
 #
 # []
 #       arg0    ֥ͥå
+#       arg1    Shared-network
 #
 # [֤]
 #       0       ե뤬¸ߤʤޤϥХ
@@ -1045,8 +1075,8 @@ sub del_dir
 
 sub check_hostsize
 {
-    my ($subnet) = @_;
-    my $host = $WORKDIR . "/" . $subnet . "/host";
+    my ($subnet, $shared) = @_;
+    my $host = $WORKDIR . "/" . $shared . "/" . $subnet . "/host";
 
     # ۥȤ¸ߥå
     if (( -e $host ) && ( !-z $host)) {
@@ -1309,9 +1339,13 @@ sub check_referer
 
     # ץ̾Ǽ
     my @linklist = ("top.cgi",
+                    "network.cgi",
+                    "sharedlist.cgi",
                     "serverlist.cgi",
                     "serverlist_dhcp.cgi",
                     "serverlist_user.cgi",
+                    "sharedlist.cgi",
+                    "sharedmod.cgi",
                     "subnetlist.cgi",
                     "subnetadd.cgi",
                     "range.cgi",
@@ -1586,7 +1620,8 @@ sub check_conv_macaddr
 # []
 #       arg0    ֥ͥå
 #       arg1    ֥ͥåȾǼ(Ϥ)
-#       arg2    顼å
+#       arg2    Shared-network̾
+#       arg3    顼å
 #
 # [֤]
 #       0       OK
@@ -1596,19 +1631,19 @@ sub check_conv_macaddr
 
 sub get_subnetinformation
 {
-    my ($subnet, $subnetdata, $errmsg) = @_;
+    my ($subnet, $subnetdata, $shared, $errmsg) = @_;
     my $subnetfile;
 
     # "/""-"ִ
     $subnet =~ s/\//-/;
 
     # ֥ͥåȥե
-    $subnetfile = $WORKDIR . "/" . $subnet . "/subnet";
+    $subnetfile = $WORKDIR . "/" . $shared . "/" . $subnet . "/subnet";
 
     # ե륪ץ
     if (open(IN, $subnetfile) == 0) {
         $$errmsg[$L_DISP] = $LANG{$ENV{$DISPLANG}}{'ERR_SUBNET_FILE1'} . "(" . $subnetfile . ")";
-        $$errmsg[$L_LOG] = $LANG{$ENV{$LOGLANG}}{'ERR_SUBNET_FILE1'} . "(" . $subnetfile . ")";
+        $$errmsg[$L_LOG] = $LANG{$LOGLANG}{'ERR_SUBNET_FILE1'} . "(" . $subnetfile . ")";
         return -1;
     }
 
@@ -1908,7 +1943,6 @@ sub dhcp_init_nosession
     return 0;
 }
 
-
 #
 # 2ʿѴ
 #
@@ -1929,4 +1963,229 @@ sub add2bin
     return sprintf("%08b%08b%08b%08b", $tmp[0], $tmp[1], $tmp[2], $tmp[3]);
 }
 
+#
+# ʬפξΡclientηåMACɥ쥹Ѵ
+#
+# []
+#       arg0    åhost̾
+#       arg1    åMACɥ쥹
+#       arg2    ѴMACɥ쥹(Ϥ)
+#       arg3    IPɥ쥹
+#       arg4    顼å
+#
+# [֤]
+#       0       OK
+#       1       顼
+#
+#
+
+sub check_client_part
+{
+    my ($hostname, $macaddr, $convmacaddr, $ipaddr, $errmsg) = @_;
+
+    # ۥ̾Υå
+    if (main::check_hostname($hostname) == 1) {
+        $$errmsg[$main::L_DISP] = $main::LANG{$main::ENV{$main::DISPLANG}}{'ERR_HOST_FORMAT'};
+        $$errmsg[$main::L_LOG] = $main::LANG{$main::LOGLANG}{'ERR_HOST_FORMAT'};
+        return 1;
+    }
+
+    # MACɥ쥹ΥåѴ
+    if (main::check_macaddr_part($macaddr, $convmacaddr, $errmsg) == 1) {
+        return 1;
+    }
+    
+    if ($ipaddr eq "") {
+        return 0;
+    }
+ 
+    # IPɥ쥹Υå
+    if (main::check_ipaddr($ipaddr, $errmsg) == 1) {
+         return 1;
+    }
+
+    return 0;
+}
+
+# MACɥ쥹ηåȳǼ
+#
+# []
+#       arg0    åMACɥ쥹̾
+#       arg1    ѴMACɥ쥹̾
+#
+# [֤]
+#       0       OK
+#       1       顼
+#
+
+sub check_macaddr_part
+{
+    my ($macaddr, $convmacaddr, $errmsg) = @_;
+    my $checkstr;
+    my @value;
+
+    # ʤ
+    if ($macaddr eq "") {
+        return 0;
+    }
+    # ѿ.:ʳʸ󤬤
+    if ($macaddr =~ /[^0-9a-fA-F\.\:]/) {
+        $$errmsg[$main::L_DISP] = $main::LANG{$main::ENV{$main::DISPLANG}}{'ERR_MACADDR_FORMAT'};
+        $$errmsg[$main::L_LOG] = $main::LANG{$main::ENV{$main::LOGLANG}}{'ERR_MACADDR_FORMAT'};
+        return 1;
+    }
+
+    # ʸ3İʾϢ³Ƥ硢ߤƤʤ
+    if ($macaddr =~ /[0-9a-fA-F]{3,}/ && $macaddr =~ /\:/) {
+        $$errmsg[$main::L_DISP] = $main::LANG{$main::ENV{$main::DISPLANG}}{'ERR_MACADDR_FORMAT'};
+        $$errmsg[$main::L_LOG] = $main::LANG{$main::ENV{$main::LOGLANG}}{'ERR_MACADDR_FORMAT'};
+        return 1;
+    }
+
+    # Ѵ
+    # ʸѴ
+    $$convmacaddr = lc($macaddr);
+
+    return 0;
+}
+
+# ꤷǥ쥯ȥΤߺ
+#
+# []
+#       arg1    ǥ쥯ȥ̾
+#       arg0    顼å
+#
+# [֤]
+#       0       OK
+#      -1       ƥ२顼
+#
+sub del_onedir
+{
+    my ($deldirpath, $errmsg) = @_;
+    my @tmpfilelist;
+    my $delfilename;
+    my $ret;
+ 
+    # ǥ쥯ȥ۲Υե
+    if (read_dir($deldirpath, \@tmpfilelist, $errmsg) < 0 ) {
+        return -1;
+    }
+
+    # եκ
+    foreach $delfilename (@tmpfilelist) {
+        $ret = unlink($delfilename);
+
+        # եκ
+        if ($ret == 0) {
+            $$errmsg[$L_DISP] = $LANG{$ENV{$DISPLANG}}{'ERR_TMP_FILE3'}
+                                . "(" . $delfilename . ")";
+            $$errmsg[$L_LOG] = $LANG{$LOGLANG}{'ERR_TMP_FILE3'}
+                                . "(" . $delfilename . ")";
+            return -1;
+        }
+    }
+
+    # ǥ쥯ȥκ
+    if (rmdir($deldirpath) == 0) { 
+        $$errmsg[$L_DISP] = $LANG{$ENV{$DISPLANG}}{'ERR_TMP_DIR2'}
+                            . "(" . $deldirpath . ")";
+        $$errmsg[$L_LOG] = $LANG{$LOGLANG}{'ERR_TMP_DIR2'}
+                           . "(" . $deldirpath . ")";
+        return -1;
+    }
+    return 0;
+}
+
+#
+# subnetɽhtmlκ
+#
+# 
+#       arg0    ɽhtml(Ϥ)
+#       arg1    򤵤줿subnet(Ϥ)
+#       arg3    顼å
+#
+# ֤
+#       0          
+#       -1        顼
+#
+
+sub make_subnethtml
+{
+    my ($subnethtml, $selectsubnet, $errmsg) = @_;
+    my %dirlist;
+    my @shareddirlist;
+    my @tmpdirlist;
+    my @value;
+    my %tmpline;
+    my $subnet;
+    my $line;
+    my $selectstr;
+    my $select;
+    my $valueline;
+    my $shared;
+    my $workdir;
+    my $subnetlist_key;
+    my $subnetlist;
+
+
+    # shared-networkǥ쥯ȥ
+    if (read_dir($WORKDIR, \@shareddirlist, $errmsg) < 0 ) {
+        return -1;
+    }
+
+    # shared-networkǥ쥯ȥʬdirlist
+    foreach $shared (@shareddirlist) {
+
+        # ǥ쥯ȥɤ߽Ф
+        if (-d $shared) {
+            if (read_dir($shared, \@tmpdirlist, $errmsg) < 0 ) {
+                return -1;
+            }
+
+            # ¤ؤbitѴ
+            conv_bit_subnet(\@tmpdirlist, \%dirlist, $errmsg);
+        }
+    }
+
+    # 
+    %tmpline = ();
+
+    # ǥ쥯ȥ̾Υ
+    foreach $subnetlist_key (sort keys %dirlist) {
+        $subnetlist = $dirlist{$subnetlist_key};
+        foreach $line (sort {$subnetlist -> {$a} cmp $subnetlist -> {$b}} keys %$subnetlist) {
+
+            # valueʸ
+            $valueline = $$subnetlist{$line};
+
+            # ɽʸ
+            if (conv_outputsubnet($valueline, \$subnet, \$shared, $errmsg) > 0) {
+                next;
+            }
+
+            # 
+            $selectstr = "";
+
+            foreach $select (@$selectsubnet) {
+                # 򤵤Ƥ
+                if ($shared . "/" . $subnet eq $select) {
+                    $selectstr = "selected";
+                    last;
+                }
+            }
+
+            $tmpline{$line} =  "<option value=" . $shared . "/" . $subnet . " " .  $selectstr . ">" .
+                          $subnet . " </option>\n";
+        }
+    }
+
+    # 
+    $$subnethtml = "";
+    foreach $line (sort {$a cmp $b} keys %tmpline) {
+        # htmlʸκ
+        $$subnethtml .= $tmpline{$line};
+    }
+
+    return 0;
+}
 1;
