@@ -23,13 +23,14 @@
 # dhcpե饤֥
 #
 # $RCSfile: libdhcpconfig.pl,v $
-# $Revision: 1.38 $
-# $Date: 2009/01/04 23:57:51 $
+# $Revision: 1.49 $
+# $Date: 2013/01/17 04:48:00 $
 #
 ##########################################################
 
 use strict;
 use IO::File;
+use Data::Dumper;
 
 ##################饤֥ꥰХ##################
 my @subnetitem = (
@@ -90,6 +91,8 @@ my $STR_SUBNET = "subnet";
 my $STR_HOST = "host";
 my $STR_RANGE = "range";
 my $STR_END = "}";
+my $STR_SUBNET = "subnet";
+my $STR_SHNETWORK = "shared-network";
 
 # åʸ(booting)
 my $STR_ALLOW_BOOTING = "allow";
@@ -131,6 +134,7 @@ sub make_common
 
 #
 # ֥ͥåκ
+# workǥ쥯ȥɤ߹ǡ֥ͥåȤ
 #
 # []
 #	arg0	
@@ -142,7 +146,6 @@ sub make_common
 #	-1	NG
 #
 #
-
 sub make_subnet
 {
     my ($conf, $fh, $errmsg) = @_;
@@ -151,10 +154,19 @@ sub make_subnet
     my $range;
     my $enhance;
     my $host;
+    my @shareddirs;
     my @dirs;
+    my @sortdirs;
+    my $shareddir_path;
+    my $shareddir;
+    my $dir_path;
     my $dir;
     my @list;
+    my @value;
     my $ret;
+    my $shared;
+    my $shared_flag;
+    my $space;
 
     # workǥ쥯ȥΥǥ쥯ȥ
     if (opendir(DIR, $WORKDIR) != 1) {
@@ -162,67 +174,99 @@ sub make_subnet
 	$$errmsg[$L_LOG] = $LANG{$LOGLANG}{'ERR_TMP_DIR1'} . "(" . $WORKDIR . ")";
 	return -1;
     }
-    @dirs = sort grep { -d "$WORKDIR/$_"} readdir(DIR);
+    @shareddirs = sort grep { -d "$WORKDIR/$_"} readdir(DIR);
     closedir(DIR);
 
-    foreach $dir(@dirs) {
+    foreach $shareddir (@shareddirs) {
+        $shared_flag = 0;
+        $space = "";
 
-	# 
-	$subnet = "";
-	$option = "";
-	$host = "";
-	$range = "";
-	$enhance = "";
+      	# ".",".."ϡ
+        if ($shareddir eq "." || $shareddir eq "..") {
+            next;
+        }
 
-	# ".",".."ϡ
-	if ($dir == "." || $dir == "..") {
-	    next;
-	}
+        if ($shareddir ne $main::OTHERDIR) {
+            $shared_flag = 1;
+            $space = "    ";
+            $shared = "shared-network " . $shareddir . " {\n";
+            print $fh $shared;
+        }
 
-	# ֥ͥåȡޥμ
-	@list = split(/-/, $dir);
-	$subnet .= $STR_SUBNET  . " " . $list[0] . " netmask " . $list[1] . " {\n";
+        $shareddir_path = $main::WORKDIR . "/" . $shareddir;
+        if (-d $shareddir_path) {
+            if (read_dir($shareddir_path, \@dirs, $errmsg) != 0 ) {
+                return -1;
+            }
 
-	# ץκ
-	$ret = make_option($dir, \$option, $errmsg);
-	if ($ret < 0) {
-	    return -1;
-	}
-	$subnet .= $option;
+            # ֥ͥåȥǥ쥯ȥ꤬¸ߤ뤫ɤ
+            @sortdirs = sort @dirs;
+            foreach $dir_path (@sortdirs) {
+                $dir_path =~ s/$WORKDIR\///g;
+                @value = split(/\//, $dir_path);
 
-	# ϰϾκ
-	$ret = make_range($dir, \$range, $errmsg);
-	if ($ret < 0) {
-	    return -1;
-	}
-	$subnet .= $range;
+                $dir = $value[1];
 
-	# ĥκ
-	$ret = make_ennanceoption($conf, $dir, \$enhance, $errmsg);
-	if ($ret < 0) {
-	    return -1;
-	}
+        	# 
+        	$subnet = "";
+        	$option = "";
+        	$host = "";
+        	$range = "";
+        	$enhance = "";
 
-	# ʤä񤭹
-	if ($enhance ne "") {
-	    $subnet .= $enhance;
-	}
+        	# ".",".."ϡ
+        	if ($dir == "." || $dir == "..") {
+        	    next;
+        	}
 
-	# Ԥ򣱹ɲ
-	$subnet .= "\n";
-
-	# ۥȾκ
-	$ret = make_host($dir, \$host, $errmsg);
-	if ($ret < 0) {
-	    return -1;
-	}
-	$subnet .= $host;
-
-	$subnet .= "}\n";
-	print $fh $subnet;
+        	# ֥ͥåȡޥμ
+        	@list = split(/-/, $dir);
+        	$subnet .= $space . $STR_SUBNET  . " " . $list[0] . " netmask " . $list[1] . " {\n";
+        
+        	# ץκ
+        	$ret = make_option($shareddir, $dir, \$option, $errmsg);
+        	if ($ret < 0) {
+        	    return -1;
+        	}
+        	$subnet .= $option;
+        
+        	# ϰϾκ
+        	$ret = make_range($shareddir, $dir, \$range, $errmsg);
+        	if ($ret < 0) {
+        	    return -1;
+        	}
+       	        $subnet .= $range;
 
+        	# ĥκ
+        	$ret = make_ennanceoption($shareddir, $conf, $dir, \$enhance, $errmsg);
+        	if ($ret < 0) {
+        	    return -1;
+        	}
+        
+        	# ʤä񤭹
+        	if ($enhance ne "") {
+        	    $subnet .= $enhance;
+        	}
+        
+        	# Ԥ򣱹ɲ
+        	$subnet .= "\n";
+        
+        	# ۥȾκ
+        	$ret = make_host($shareddir, $dir, \$host, $errmsg);
+        	if ($ret < 0) {
+        	    return -1;
+        	}
+        
+        	$subnet .= $host;
+        
+        	$subnet .= $space . "}\n";
+        	print $fh $subnet;
+            }
+            if ($shared_flag == 1) { 
+                print $fh "}\n";
+            }
+        }
     }
-
     return 0;
 }
 
@@ -230,9 +274,10 @@ sub make_subnet
 # ץκ
 #
 # []
-#	arg0	֥ͥåȥǥ쥯ȥ̾
-#	arg1	֥ͥåȾ(Ϥ)
-#	arg2	顼å
+#	arg0	shared-networkǥ쥯ȥ̾
+#	arg1	֥ͥåȥǥ쥯ȥ̾
+#	arg2	֥ͥåȾ(Ϥ)
+#	arg3	顼å
 #
 # [֤]
 #	0	OK
@@ -242,8 +287,8 @@ sub make_subnet
 
 sub make_option
 {
-    my ($dir, $option, $errmsg) = @_;
-    my $option_file = $WORKDIR . "/" . $dir . "/subnet" ;	# ֥ͥåȥե
+    my ($shared, $dir, $option, $errmsg) = @_;
+    my $option_file = $WORKDIR  . "/" . $shared . "/" . $dir . "/subnet" ;	# ֥ͥåȥե
     my $n = 0;
 
     # ץեɹ
@@ -279,14 +324,14 @@ sub make_option
 
     return 0;
 }
-
 #
 # ϰϾκ
 #
 # []
-#	arg0	֥ͥåȥǥ쥯ȥ̾
-#	arg1	ϰ(Ϥ)
-#	arg2	顼å
+#	arg0    shared-networkǥ쥯ȥ̾
+#	arg1	֥ͥåȥǥ쥯ȥ̾
+#	arg2	ϰ(Ϥ)
+#	arg3	顼å
 #
 # [֤]
 #	0	OK
@@ -296,8 +341,8 @@ sub make_option
 
 sub make_range
 {
-    my ($dir, $range, $errmsg) = @_;
-    my $range_file = $WORKDIR . "/" . $dir . "/range" ;	# 󥸥ե
+    my ($shared, $dir, $range, $errmsg) = @_;
+    my $range_file = $WORKDIR . "/" . $shared . "/" . $dir . "/range" ;	# 󥸥ե
     my @list;
 
     # rangeե뤬¸ߤɤ߹
@@ -337,10 +382,11 @@ sub make_range
 # ĥץκ
 #
 # []
-#	arg0	
-#	arg1	֥ͥåȥǥ쥯ȥ̾
-#	arg2	ĥץ(Ϥ)
-#	arg3	顼å
+#	arg0	shared-networkǥ쥯ȥ̾
+#	arg1	
+#	arg2	֥ͥåȥǥ쥯ȥ̾
+#	arg3	ĥץ(Ϥ)
+#	arg4	顼å
 #
 # [֤]
 #	0	OK
@@ -350,8 +396,8 @@ sub make_range
 
 sub make_ennanceoption
 {
-    my ($conf, $dir, $ennanceoption, $errmsg) = @_;
-    my $ennance_file = $WORKDIR . "/" . $dir . "/option" ;	# ĥץե
+    my ($shared, $conf, $dir, $ennanceoption, $errmsg) = @_;
+    my $ennance_file = $WORKDIR . "/" . $shared . "/" . $dir . "/option" ;	# ĥץե
     my @list;
     my $option;
 
@@ -395,8 +441,8 @@ sub make_ennanceoption
 
 sub make_host
 {
-    my ($dir, $host, $errmsg) = @_;
-    my $host_file = $WORKDIR . "/" . $dir . "/host" ;	# 󥸥ե
+    my ($shared, $dir, $host, $errmsg) = @_;
+    my $host_file = $WORKDIR . "/" . $shared . "/" . $dir . "/host" ;	# 󥸥ե
     my @list;
 
     # ۥȾɹ
@@ -509,6 +555,8 @@ sub analyze_dhcpd_conf
     my $fd;
     my $subnetline;
     my $optionname = "";
+    my $newlinenum;
+    my @subnetdata;
 
     # 켡եκ
     if (del_tmpfile($errmsg) < 0 ) { 
@@ -536,10 +584,33 @@ sub analyze_dhcpd_conf
 
     # subnetǡ
     if (eof($fd) != 1) {
+        
+        # subnetǡ˳Ǽ
+        chomp($subnetline);
+        push(@subnetdata, $subnetline);
 
-        # subnetǥ쥯ȥ
-        read_subnet($fd, \$linenum, \$subnetline, \$checkflag, $optionname, 
-                    $errmsg);
+        while (eof($fd) == 0) {
+
+            # 1Ԥɤ
+            $line = <$fd>;
+
+            # Ժ
+            chomp($line);
+
+            # Ƭζ򡢥֤
+            $line =~ s/^\s+//g;
+
+            # Ф
+            if ($line eq "") {
+                 next;
+            }
+
+            push(@subnetdata, $line);
+        }
+
+        # ƥǥ쥯ȥ
+        $linenum = 0;
+        check_subnet_line(\@subnetdata, $linenum, $optionname, $errmsg); 
     } 
 
     close(IN);
@@ -568,24 +639,24 @@ sub read_common
     my $commonline;
     my $commonfile;
     my $subnetstr;  # åʸ
+    my $shnetworkstr;  # åʸ
 
     # åʸκ
     $subnetstr = '^' . $STR_SUBNET . '\s+.*{$';
+    $shnetworkstr = '^' . $STR_SHNETWORK . '\s+.*{$';
 
     # 
     $commonline = "";
 
-    for (;($line !~ /$subnetstr/) && (eof($fd) != 1); $line = <$fd>) {
+    for (;($line !~ /$subnetstr/) && ($line !~ /$shnetworkstr/) && (eof($fd) != 1); $line = <$fd>) {
         $$linenum++;
 
         # ɵ
         $commonline .= $line;
     }
 
-    # subnet饤Ǽ
-    if ($line !~ /$subnetstr/) {
-        $commonline .= $line;
-    } else {
+    # commonʲǼ
+    if ($line =~ /$subnetstr/ || $line =~ /$shnetworkstr/) {
         $$subnetline = $line;
     }
 
@@ -598,97 +669,6 @@ sub read_common
     return 0;
 }
 
-#
-# subnetɤ߹
-#
-# 
-#	arg0	եΥǥץ
-#	arg1	ԿʻϤ
-#	arg2	subnetΰܡʻϤ
-#	arg3	顼ѥե饰(:0,顼:1)
-#	arg4	ĥץ
-#	arg5	顼å
-#
-# ֤
-#	   0	
-#
-
-sub read_subnet 
-{
-    my ($fd, $linenum, $firstsubnetline, $checkflag, $optionname, $errmsg) = @_;
-    my $line;
-    my $sknum = 1;            # Ϥޤγ̤ο subnetϤޤäƤ
-    my $eknum = 0;            # γ̤ο
-    my @subnetline;
-    my $slinenum = $$linenum; # subnetκǽιԿǼ
-    my $ret;
-    my $subnetdirname;
-    my $subnetstr;            # åʸ
-    my $hoststr; 
-
-    # åʸκ
-    $subnetstr = '^' . $STR_SUBNET . '\s+.*{$';
-    $hoststr = '^' . $STR_HOST . '\s+.*{$';
-
-    # subnet ..{ ޤǤޤ魯
-    for (;($line !~ /$subnetstr/) && (eof($fd) != 1);) {
-
-        # 1ɤ߹
-        $line = <$fd>;
-
-        # Կ
-        $$linenum++;
-
-        # Ƭζ򡢥֤
-        $line =~ s/^\s+//g;
-
-        # Ժ
-        chomp($line);
-
-        # host start å
-        if ($line =~ /$hoststr/) {
-            $sknum++;
-        }
-
-        # subnet host end å
-        if ($line =~ /^}$/) {
-            $eknum++;
-        }
-
-        # subnetι԰ʳ
-        if ($line !~ /$subnetstr/) {
-            push(@subnetline, $line);
-        }
-    }
-
-    # subnetå(1)
-    if (check_subnet_init($$firstsubnetline, $slinenum, $sknum, $eknum, 
-                          \$subnetdirname, $errmsg) > 0) {
-
-        # ̵subnetξ
-        $$checkflag = 1;
-    } else {
-
-        # subnet,hostΰ켡ե
-        $ret = make_subnet_host_data($slinenum, $subnetdirname, 
-                                     \@subnetline, $optionname, $errmsg);
-        # 顼ä
-        if ($ret != 0) {
-            $$checkflag = 1;
-        }
-    }
-
-    # subnet
-    if ($line =~ /$subnetstr/) {
-        $$firstsubnetline = $line;
-        # subnetɤ 
-        read_subnet($fd, $linenum, $firstsubnetline, $checkflag, $optionname, 
-                    $errmsg);
-    }
-
-    return 0;
-}
-
 #
 # 켡եκ
 #
@@ -733,7 +713,6 @@ sub del_tmpfile
 
         # ǥ쥯ȥξ
         if ( -d $filepath) {
-
             # ǥ쥯ȥ
             if (del_dir($filepath, $errmsg) < 0) {
                 return -1;
@@ -744,237 +723,13 @@ sub del_tmpfile
     return 0;
 }
 
-#
-# 켡ǥ쥯ȥꡢեκ
-#
-# 
-#	arg0	subnetκǽιԿ
-#	arg1	ǥ쥯ȥ̾
-#	arg2	subnetιԥǡ()
-#	arg3	ĥץ
-#	arg4	顼å
-#
-# ֤
-#	 0	
-#	 1	顼
-#	 2	顼(subnet̵)
-#	-1	ƥ२顼
-#
-
-sub make_subnet_host_data 
-{
-    my ($linenum, $subnetdirname, $subnetline, $optionname, $errmsg) = @_;
-    my %subnetdata;
-    my %hostdata;
-    my @allhostdata;
-    my $flag = 0;     # returnѥե饰
-    my $ret;
-    my $subnetdirpath;
-    my $inputhostline;
-
-    # subnet,hostǡ
-    $ret = get_subnet_host_data($linenum, \%subnetdata, \@allhostdata, 
-                                $subnetline, $optionname, $errmsg); 
-
-    # subnet̵ͤʾ
-    if ($ret == 2) {
-        return 2;
-    }
-
-    # subnetԤ
-    if ($ret == 1) {
-        $flag = 1;
-    }
-
-    # subnetǥ쥯ȥ̾κ
-    $subnetdirpath = $WORKDIR . "/" . $subnetdirname;
-    
-    # ɬܥå(subnet)
-    if (check_subnetitem(\%subnetdata, $subnetdirname, $errmsg) > 0) {
-        return 2;
-    }
-
-    # ǥ쥯ȥ
-    if (make_dir($subnetdirpath, $errmsg) < 0) {
-        return -1;
-    }
-
-    # 켡ե(subnet, range)
-    if (write_subnetitem($subnetdirname, \%subnetdata, $optionname, $errmsg) 
-        < 0 ) {
-        return -1;
-    }
-
-    # hostǡθĿ
-    if (check_hostitem(\@allhostdata, \$inputhostline, $subnetdirname, 
-                       $errmsg) > 0 ) {
-        # 1ĤǤ顼ä
-        $flag = 1;
-    }
-
-    # 켡ե(host)
-    if (write_hostitem($subnetdirname, $inputhostline, $errmsg) < 0 ) {
-        return -1;
-    }
-
-    return $flag;
-}
-
-#
-# subnet,hostΥǡΥåȳǼ
-#
-# 
-#	arg0	subnetκǽιԿ
-#	arg1	subnetΥǡ(Ϣ)
-#	arg2	subnethostǡ()
-#	arg3	subnetιԥǡ()
-#	arg4	ĥץ
-#	arg5	顼å
-#
-# ֤
-#	0	
-#	1	subnet顼orhost̵orhost顼
-#	2	顼(subnet̵)
-#
-
-sub get_subnet_host_data 
-{
-    my ($linenum, $subnetdata, $allhostdata, $subnetalldata, $optionname,
-        $errmsg) = @_;
-    my $status = 1;  #ơ        0: subnet
-                     #                  1: subnet
-                     #                  2: host
-    my @hostline;
-    my %hostdata;
-    my $checkline;
-    my $slinenum = $linenum;
-    my $hnum = 0;
-    my $ret;
-    my $starthostline;
-    my $checkflag = 0;    # ֤ѥե饰
-    my $hflag = 0;        # hostѤΥե饰
-    my $hnum = 0;         # hostθĿ
-    my $starthoststr;     # åʸ
-    my $endhoststr;       # åʸ
-
-    # åʸκ
-    $starthoststr = '^' . $STR_HOST . '\s+.*{$';
-    $endhoststr =  '^' . $STR_END . '$';
-
-    foreach $checkline (@$subnetalldata) {
-
-        # Կ
-        $slinenum++;
-
-        # host {
-        if ($checkline =~ /$starthoststr/i) {
-            # ǽγ̤ξ
-            if ($status == 1) {
-                # hostϤޤԿ
-                $starthostline = $slinenum;
-            }
-            $status++;
-        }
-
-        # }
-        if ($checkline =~ /}$/i) {
-            # ơ򸺤餹
-            $status--;
-
-            # host}ɤ
-            if ($status == 1) {
-                $hflag = 1;
-            } else {
-                next;
-            }
-        }
-
-        # host, hostlineɵ
-        if ($status >= 2) {
-            push(@hostline, $checkline);
-            next;
-        }
-
-        # hostä
-        if ($hflag == 1) {
-
-            # hostǡγǼ
-            if (get_all_host_data($starthostline, $allhostdata, 
-                                  \@hostline, \$hnum, $errmsg) > 0) {
-                $checkflag = 1;
-            }
-
-            # 
-            undef(@hostline);
-            $hflag = 0;
-            next;
-        }
-
-        # subnet
-        if ($status == 1) {
-            # ,Ȥξ
-            if ($checkline =~ /^$/ || $checkline =~ /^#/) {
-                next;
-            }
-
-            # ĥץΥåǼ
-            if ($optionname ne "") {
-                $ret = check_subnetline_ext($checkline, $slinenum, $subnetdata
-                                            , $optionname, $errmsg);
-                # ʣ顼ξ
-                if($ret == 2) {
-                    return 2;
-                 }
-
-                # ĥץ󤬤
-                if ($ret == 0) {
-                    next;
-                }
-            }
-
-            # subnetԥå,Ǽ
-            $ret = check_subnetline($checkline, $slinenum, $subnetdata, 
-                                    $errmsg);
-            # ʣ顼ξ
-            if($ret == 2) {
-                return 2;
-
-            # 顼ξ
-            } elsif ($ret == 1) {
-                $checkflag = 1;
-            }
-            next;
-        }
-
-        # subnetν
-        if ($status == 0) {
-
-            # ,Ȥξ
-            if ($checkline =~ /^$/ || $checkline =~ /^#/) {
-                next;
-            }
-
-            # Ԥξ
-            $$errmsg[$L_DISP] .= $LANG{$ENV{$DISPLANG}}{'ERR_DHCPD_CONF_FORM'} 
-                                . "(" . $LANG{$ENV{$DISPLANG}}{'ERR_LINE_STR'}
-                                . $slinenum . ")\n";
-            $$errmsg[$L_LOG] .= $LANG{$LOGLANG}{'ERR_DHCPD_CONF_FORM'} 
-                               . "(" . $LANG{$LOGLANG}{'ERR_LINE_STR'} 
-                               . $slinenum . ")\n";
-            $checkflag = 1;
-        }
-    }
-
-    return $checkflag;
-}
-
 #
 # subnetƤhostФ,Ǽ
 #
 # 
-#	arg0	subnetκǽι
-#	arg1	subnetκǽιԿ
-#	arg2	̤ο
+#	arg0	hostκǽι
+#	arg1	hostξ Ϥ
+#	arg2	ϤäƤhostξ ()
 #	arg3	֥ͥåȥǥ쥯ȥ̾ʻϤ
 #	arg4	hostοʻϤ
 #	arg5	顼åʻϤ
@@ -987,6 +742,7 @@ sub get_subnet_host_data
 sub get_all_host_data 
 {
     my ($starthostline, $allhostdata, $hostline, $hnum, $errmsg) = @_;
+    my @hostline = @{$hostline};
     my %hostdata;
     my $key;
     my $value;
@@ -1012,61 +768,12 @@ sub get_all_host_data
     return $flag;
 }
 
-#
-# subnetνå
-#
-# 
-#	arg0	subnetκǽι
-#	arg1	subnetκǽιԿ
-#	arg2	Ϥޤγ̤ο
-#	arg3	̤ο
-#	arg4	֥ͥåȥǥ쥯ȥ̾ʻϤ
-#	arg5	顼åʻϤ
-#
-# ֤
-#	 0	
-#	 1	顼(subnet̵)
-#
-
-sub check_subnet_init 
-{
-    my ($firstline, $linenum, $sknum, $eknum, $subnetdirname, $errmsg) = @_;
-    my @subnet;
-
-    # subnet̾å
-    @subnet = split(/\s+/, $firstline);
-    if ($#subnet != 4) {
-        $$errmsg[$L_DISP] .= $LANG{$ENV{$DISPLANG}}{'ERR_SUBNET_DATA2'}
-                             . "(" . $LANG{$ENV{$DISPLANG}}{'ERR_LINE_STR'} 
-                             . $linenum . ")\n";
-        $$errmsg[$L_LOG] .= $LANG{$ENV{$DISPLANG}}{'ERR_SUBNET_DATA2'}
-                            . "(" . $LANG{$LOGLANG}{'ERR_LINE_STR'} 
-                            . $linenum . ")\n";
-        return 1;
-    }
-
-    # subnetǥ쥯ȥ̾κ
-    $$subnetdirname = sprintf("%s-%s", $subnet[1], $subnet[3]);
-
-    # ̤бå
-    if ($sknum != $eknum) {
-        $$errmsg[$L_DISP] .= $LANG{$ENV{$DISPLANG}}{'ERR_SUBNET_DATA2'}
-                            . "(" . $$subnetdirname . ")\n";
-        $$errmsg[$L_LOG] .= $LANG{$LOGLANG}{'ERR_SUBNET_DATA2'}
-                           . "(" . $$subnetdirname . ")\n";
-        return 1;
-    }
-
-    return 0;
-}
-
 #
 # subnetǡγǼ(ĥץ)
 #
 # 
 #	arg0	åsubnetι
-#	arg1	subnetιԿ
-
+#	arg1	subnetιԿ (顼å)
 #	arg2	ǼsubnetǡϢ
 #	arg3	ĥץ
 #	arg4	顼å(Ϥ)
@@ -1103,7 +810,9 @@ sub check_subnetline_ext
         chop($subnetline);
 
         # ʬִ
+        # ɤ߹֥ͥåȤιԤ
         $$subnetdata{$optionname} = $subnetline . "\n";
+        # ꤵ줿ĥץ
         $$subnetdata{$optionname} =~ s/$optionname //g;
 
         return 0;
@@ -1119,7 +828,7 @@ sub check_subnetline_ext
 #
 # 
 #	arg0	åsubnetι
-#	arg1	subnetιԿ
+#	arg1	subnetιԿ (顼åѡ
 #	arg2	ǼsubnetǡϢ
 #	arg3	顼å(Ϥ)
 #
@@ -1141,12 +850,12 @@ sub check_subnetline
         # åʸκ
         $checkstr = '^' . $sitem . '\s+.*;$';
 
-        # פʤä
+        # פʤä顢ιܤ
         if ($subnetline !~ /$checkstr/i) {
             next;
         }
 
-        # ʣå
+        # ʣå rangeä̵
         if (defined($$subnetdata{$sitem}) && $sitem ne $subnetitem[$RANGE]) {
             $$errmsg[$L_DISP] .= $LANG{$ENV{$DISPLANG}}{'ERR_DHCPD_LINE2'}
                                  . "(" . $LANG{$ENV{$DISPLANG}}{'ERR_LINE_STR'}
@@ -1163,17 +872,19 @@ sub check_subnetline
         # ڤ
         @value = split(/\s+/, $subnetline);
 
-        # 顼ξ
+        # 顼ξ ڤΥå
         if ($subnetinfo{$sitem}(@value) > 0) {
             last;
         }
 
+        # Τ
         $checkflag = 1;
 
         # rangeξ
         if ($sitem eq $subnetitem[$RANGE]) {
 
             # dynamic-bootpʤ
+            # (ब2Ĥ2ܤdynamic-bootpǤϤʤ)
             if ($#value == 2 && $value[1] ne $DYNAMIC_BOOTP) {
 
                 # ϢǼ
@@ -1189,10 +900,11 @@ sub check_subnetline
                                                  $value[3], 
                                                  $value[1]);
             } else {
-                # rangeη
+                # rangeη 顼
                 $checkflag = 0;
                 last;
             }
+        # rangeǤ
         } else {
 
             # ϢǼ
@@ -1224,7 +936,7 @@ sub check_subnetline
 #
 # 
 #	arg0	hostκǽιԿ
-#	arg1	hostΥǡ()
+#	arg1	hostΥǡ( Ϥ)
 #	arg2	hostι(
 #	arg3	顼å
 #
@@ -1251,7 +963,6 @@ sub get_host_data
     # åʸκ
     $hoststr = '^' . $STR_HOST . '\s+.*{$';
 
-
     foreach $checkline (@$hostalldata) {
 
         # Կ
@@ -1260,15 +971,16 @@ sub get_host_data
         # ܥåѥե饰
         $checkflag = 0;
 
-        # ,Ȥξ
+        # ,Ȥξ ιԤ
         if ($checkline =~ /^$/ || $checkline =~ /^#/) {
             next;
         }
 
+        # hosthost񤫤Ƥʤå
         # host {
         if ($checkline =~ /$hoststr/i) {
             $status++;
-            # 2Ĥhost {
+            # 2Ĥhost { ۥȤηʤΤǥ顼
             if ($status > 1) {
                 $$errmsg[$L_DISP] .= $LANG{$ENV{$DISPLANG}}{'ERR_HOST_DATA1'}
                                      . "(" . $LANG{$ENV{$DISPLANG}}{'ERR_LINE_STR'}
@@ -1285,11 +997,12 @@ sub get_host_data
         $ret = check_hostline($checkline, $hlinenum, $hostdata, $errmsg);
         if ($ret == 2) {
 
-            # host̵
+            # host̵ ܤʣƤ
             return 2;
+
         } elsif ($ret == 1) {
 
-            # Ԥ
+            # ʷԤ
             $flag = 1;
         }
     }
@@ -1302,7 +1015,7 @@ sub get_host_data
 #
 # 
 #	arg0	åhostι
-#	arg1	hostιԿ
+#	arg1	hostιԿ (顼åѡ
 #	arg2	hostǡϢ
 #	arg3	顼å
 #
@@ -1350,7 +1063,7 @@ sub check_hostline
         chop($hostline);
         @value = split(/\s+/, $hostline);
 
-        # å
+        # å :ǡθĿå
         if ($hostinfo{$hitem}(@value) > 0) {
             last;
         }
@@ -1360,11 +1073,11 @@ sub check_hostline
 
         # bootingξ
         if ($hitem eq $hostitem[$OPTION_BOOTING]) {
-            # Ǽ
+            # ǡƬʬǼ allow/denyʬ
             $$hostdata{$hitem} = shift(@value);
         } else {
 
-            # Ǽ
+            # ǡκǸǼ 줾MACɥ쥹ʤ
             $$hostdata{$hitem} = pop(@value);
 
             # hostnameξ,""
@@ -1375,6 +1088,7 @@ sub check_hostline
         }
     }
 
+    # פܤʤä
     if ($checkflag == 0) {
 
         # ʹܤξ
@@ -1475,7 +1189,7 @@ sub check_subnetitem
     # ƹܤɬܥå
     foreach $item (@subnetitem) {
 
-       # rangeɬܥåʤ
+       # rangeɬܤǤϤʤΤǥåʤ
        if ($item eq $STR_RANGE) {
            next;
        }
@@ -1520,7 +1234,7 @@ sub write_subnetitem
     if ($$subnetdata{$optionname} ne "") {
 
         # 켡ե̾κ(subnet)
-        $optionfile =  $WORKDIR . "/" . $subnetdirname . "/option";
+        $optionfile =  $subnetdirname . "/option";
 
         #optionե
         if (write_file($optionfile, $$subnetdata{$optionname}, $errmsg) == -1)
@@ -1548,7 +1262,7 @@ sub write_subnetitem
     }
 
     # 켡ե̾κ(subnet)
-    $subnetfile =  $WORKDIR . "/" . $subnetdirname . "/subnet";
+    $subnetfile =  $subnetdirname . "/subnet";
 
     # 켡եκ(subnet)
     if (write_file($subnetfile, $subnetline, $errmsg) < 0 ) {
@@ -1556,7 +1270,7 @@ sub write_subnetitem
     }
 
     # 켡ե̾κ(range)
-    $rangefile =  $WORKDIR . "/" . $subnetdirname . "/range";
+    $rangefile =  $subnetdirname . "/range";
 
     # 켡եκ(range)
     if (write_file($rangefile, $$subnetdata{"range"}, $errmsg) < 0 ) {
@@ -1585,7 +1299,7 @@ sub write_hostitem
     my $hostfile;
 
     # 켡ե̾κ(host)
-    $hostfile =  $WORKDIR . "/" . $subnetdirname . "/host";
+    $hostfile = $subnetdirname . "/host";
 
     # 켡եκ(host)
     if (write_file($hostfile, $hostline, $errmsg) < 0 ) {
@@ -1682,4 +1396,582 @@ sub check_datanum3_4
 
     return 0;
 }
+
+#
+# subnetåؿ
+#
+# 1ܤsubnetɤޤ줿硢
+# Shared-networkñȤsubnetȽ̤ؿ
+#
+# []
+#       arg1    subnetǡä
+#       arg2    Կ
+#       arg3    ĥץ 
+#       arg4    顼å
+#
+# [֤]
+#       0       OK
+#       1       顼
+#
+
+sub check_subnet_line
+{
+    my ($data, $linenum, $optionname, $errmsg) = @_;
+    my @subnetdata = @{$data};
+    my $line;
+    my $num = 0;
+    my $arraynum = 0;
+    my $newlinenum = 0;
+    my $subnetstr;
+    my $shnetworkstr;
+    my $checkline;
+    my $shared_flag = 0;
+    my @copy_subnetdata;
+    my $other_flag = 0;
+    my $tmplinenum;
+
+    # åʸκ
+    $subnetstr = '^' . $STR_SUBNET . '\s+.*{$';
+    $shnetworkstr = '^' . $STR_SHNETWORK . '\s+.*{$';
+
+    # 򥳥ԡ
+    @copy_subnetdata = @subnetdata;
+
+    # νޤǲ
+    foreach $checkline(@copy_subnetdata) {
+        # ԤȽ̤
+        # Shared-networkä
+        if ($checkline =~ /$shnetworkstr/) {
+            # Shared-networkΥǥ쥯ȥؿϤ
+            if ($other_flag == 1) {
+                make_shared_network_dir(\@subnetdata,$newlinenum,\$newlinenum, 
+                                        $optionname, $errmsg);
+                $other_flag = 0;
+            } else {
+                make_shared_network_dir(\@subnetdata,$linenum,\$newlinenum, 
+                                        $optionname, $errmsg);
+            }
+
+            # Shared-networkäե饰ΩƤ
+            $shared_flag = 1;
+
+        # ñȤsubnetä
+        } else {
+
+            if ($linenum == $newlinenum) {
+                 $shared_flag = 0;
+            }
+
+            if ($shared_flag == 0 && $subnetdata[$linenum] =~ /$subnetstr/) {
+                # _otherΥǥ쥯ȥؿϤ
+                make_other_dir(\@subnetdata, $linenum, \$newlinenum, 
+                               $optionname, $errmsg);
+
+                # _otherե饰򤿤Ƥ
+                $other_flag = 1;
+            }
+        } 
+        $linenum++;
+    }
+
+    return 0;
+}
+
+#
+# _otheråؿ
+#
+# Shared-networkʳιԤɤߤޤ줿
+# subnetɤ߹ؿ
+#
+# []
+#       arg1    subnetǡ
+#       arg2    subnetϤޤä
+#       arg3    subnetä (Ϥ
+#       arg4    ĥץ
+#       arg4    顼å(Ϥ)
+#
+# [֤]
+#       0       OK
+#       1       顼
+#
+
+sub make_other_dir 
+{
+    my ($data, $linenum, $newlinenum, $optionname, $errmsg) = @_;
+    my @subnetdata = @{$data};
+    my $line;           # ʸ
+    my $num = 0;        
+    my $subnetstr;      # åʸ ֥ͥå
+    my $sknum = 0;      # Ϥޤ̤ο
+    my $eknum = 0;      # Ĥ̤ο
+    my $shname;
+    my $dirname;
+    my $dirpath = "";
+    my $subnetnum;
+    my $tmpsubnet;
+    my @tmpshname;
+    my @subnetline;
+    my @tmpsubnetname;
+    my $eqnum = 0;
+
+    # ǥ쥯ȥ
+    # ǥ쥯ȥ꤬ʤä硢˥ǥ쥯ȥ
+    $dirpath = $WORKDIR . "/" . $OTHERDIR;
+    if (! -d $dirpath) {
+        if (make_dir($dirpath, $errmsg) < 0) {
+            return -1;
+        }
+    }
+
+    # ƬιԤޤǼ
+    while ($linenum != $num) {
+        shift @subnetdata;
+        $num++;
+    }
+
+    foreach $line (@subnetdata) {
+
+        # Ԥ򥫥
+        $num++;
+ 
+        # Ϥޤ̤򥫥
+        if ($line =~ /{/) {
+            $sknum++;
+        }
+
+        # Ĥ̤򥫥
+        if ($line =~ /^}$/) {
+            $eknum++;
+        }
+
+        push(@subnetline, $line);
+
+        # ϤȽγ̤ʤä롼פȴ
+        if ($sknum == $eknum) {
+            # ιԤ֤
+            $$newlinenum = $num;
+            last;
+        }
+    }
+
+    # ǥ쥯ȥѥ
+    $tmpsubnet = $subnetdata[0];
+    @tmpsubnetname = split(/\s+/, $tmpsubnet);
+    $dirpath .= "/" . sprintf("%s-%s", $tmpsubnetname[1], $tmpsubnetname[3]);
+    
+    # 򥵥֥ͥåȤؿϤ : 󡡥ǥ쥯ȥ̾ 
+    if (make_subnet_host_data(\@subnetline, $dirpath, $optionname, $errmsg) < 0) {
+        return -1;
+    }
+
+    return 0;
+
+}
+
+#
+# Shared-networkåؿ
+#
+# Shared-networkιԤɤߤޤ줿
+# Shared-networkɤ߹ؿ
+#
+# []
+#       arg1    subnetǡ
+#       arg2    Shared-networkϤޤä
+#       arg3    Shared-networkä (Ϥ
+#       arg4    ĥץ
+#       arg5    顼å(Ϥ)
+#
+# [֤]
+#       0       OK
+#       1       顼
+#
+
+sub make_shared_network_dir 
+{
+    my ($data, $linenum, $newlinenum, $optionname, $errmsg) = @_;
+    my @subnetdata = @{$data};
+    my $line;           # ʸ
+    my $num = 0;        
+    my $subnetstr;      # åʸ ֥ͥå
+    my $shnetworkstr;   # åʸ Shared-network
+    my $sknum = 0;      # Ϥޤ̤ο
+    my $eknum = 0;      # Ĥ̤ο
+    my $shname;
+    my $dirname;
+    my $dirpath;
+    my $subnetnum;
+    my $tmpshname;
+    my @shnetworkline;
+    my $eqnum = 0;
+
+    # åʸκ
+    $subnetstr = '^' . $STR_SUBNET . '\s+.*{$';
+    $shnetworkstr = '^' . $STR_SHNETWORK . '\s+.*{$';
+
+    # ǥ쥯ȥ̾ȴ
+    $tmpshname = $subnetdata[$linenum];
+    $tmpshname =~ s/$STR_SHNETWORK\s+(.+)\s*{/$1/g;
+    $tmpshname =~ s/^\s*(.*?)\s*$/$1/;
+
+    # ǥ쥯ȥ
+    # ǥ쥯ȥ꤬ʤä硢˥ǥ쥯ȥ
+    $dirpath = $WORKDIR . "/" . $tmpshname;
+    if (! -d $dirpath) {
+        if (make_dir($dirpath, $errmsg) < 0) {
+            return -1;
+        }
+    }
+
+    # ƬιԤޤǼ
+    while ($linenum != $num) {
+        shift @subnetdata;
+        $num++;
+    }
+
+    # Shared-networkˤ롣
+    foreach $line (@subnetdata) {
+
+        # Ԥ򥫥
+        $num++;
+ 
+        # Ϥޤ̤򥫥
+        if ($line =~ /{/) {
+            $sknum++;
+        }
+
+        # Ĥ̤򥫥
+        if ($line =~ /}/) {
+            $eknum++;
+        }
+
+        push(@shnetworkline, $line);
+
+        # ϤȽγ̤ʤä
+        if ($sknum == $eknum) {
+            # ֥ͥåȤؿϤ
+            if (make_shnetwork_subnet_dir(\@shnetworkline, $dirpath,
+                                          $optionname, $errmsg) < 0) {
+                return -1;
+            }
+
+            $$newlinenum = $num;
+            last;
+        }
+    }
+
+    return 0;
+}
+
+#
+# subnetåؿ
+#
+# Shared-networkΥ֥ͥåȤʬ
+# ֥ͥåȤΥǥ쥯ȥؿϤؿ
+# []
+#       arg0    ֥ͥåȤιԡ
+#       arg1    Shared-network̾
+#       arg2    ĥץ
+#       arg3    顼å
+#
+# [֤]
+#       0       OK
+#       1       顼
+#
+sub make_shnetwork_subnet_dir
+{
+    my ($data, $shdirname, $optionname, $errmsg) = @_;
+    my @shsubnetstr = @{$data};
+    my @subnetstr;
+    my $dirname;
+    my @tmpsubnetname;
+    my $tmpstr;
+    my $subnetflag = 0;
+    my $sknum = 0;
+    my $eknum = 0;
+    my $subnetstr;
+ 
+    $subnetstr = '^' . $STR_SUBNET . '\s+.*{$';
+
+    # κǽȺǸ
+    shift @shsubnetstr;
+    pop @shsubnetstr;
+
+    # Shared-networkȤԤɤ
+    foreach $tmpstr (@shsubnetstr) {
+
+        # subnet
+        if ($tmpstr =~ /$subnetstr/) {
+            # ǥ쥯ȥ̾
+            @tmpsubnetname = split(/\s+/, $tmpstr);
+            $dirname = $shdirname . "/" .
+                      sprintf("%s-%s", $tmpsubnetname[1], $tmpsubnetname[3]);
+            $subnetflag = 1;
+        }
+
+        if ($subnetflag == 1) {
+            push (@subnetstr, $tmpstr);
+        }    
+
+        # Ϥޤ̤򥫥
+        if ($tmpstr =~ /{/) {
+            $sknum++;
+        }
+
+        # Ĥ̤򥫥
+        if ($tmpstr =~ /}/) {
+            $eknum++;
+        }
+
+        # ̤ʤä顢֥ͥåȺؿϤ
+        if ($sknum == $eknum) {
+            if (make_subnet_host_data(\@subnetstr, $dirname, 
+                                      $optionname, $errmsg) < 0) {
+                return -1;
+            }
+
+            # ֥ͥåȤ
+            @subnetstr = ();
+
+            # ֥ͥåȤΥե饰0᤹
+            $subnetflag = 0;
+
+            # ǥ쥯ȥ̾
+            $dirname = "";
+        }            
+    }
+
+    return 0;
+}
+
+#
+# 켡ǥ쥯ȥꡢեκ
+#
+# 
+#	arg0	subnetιԡ
+#	arg1	ǥ쥯ȥ̾
+#	arg2	ĥץ
+#	arg4	顼å
+#
+# ֤
+#	 0	
+#	 1	顼
+#	 2	顼(subnet̵)
+#	-1	ƥ२顼
+#
+
+sub make_subnet_host_data 
+{
+    my ($data, $subnetdirname, $optionname, $errmsg) = @_;
+    my @subnetdata = @{$data};
+    my %subnetdata;
+    my %hostdata;
+    my @allhostdata;
+    my $flag = 0;     # returnѥե饰
+    my $ret;
+    my $linenum;
+    my $inputhostline;
+
+    # subnet,hostǡ˳Ǽ
+    $ret = get_subnet_host_data($linenum, \@subnetdata, \%subnetdata, \@allhostdata, 
+                                $optionname, $errmsg); 
+
+    # subnet̵ͤʾ
+    if ($ret == 2) {
+        return 2;
+    }
+
+    # subnetԤ
+    if ($ret == 1) {
+        $flag = 1;
+    }
+    
+    # ɬܥå(subnet)
+    if (check_subnetitem(\%subnetdata, $subnetdirname, $errmsg) > 0) {
+        return 2;
+    }
+
+    # ǥ쥯ȥ
+    if (make_dir($subnetdirname, $errmsg) < 0) {
+        return -1;
+    }
+
+    # 켡ե(subnet, range)
+    if (write_subnetitem($subnetdirname, \%subnetdata, $optionname, $errmsg) 
+        < 0 ) {
+        return -1;
+    }
+
+    # hostǡθĿ
+    if (check_hostitem(\@allhostdata, \$inputhostline, $subnetdirname, 
+                       $errmsg) > 0 ) {
+        # 1ĤǤ顼ä
+        $flag = 1;
+    }
+
+    # 켡ե(host)
+    if (write_hostitem($subnetdirname, $inputhostline, $errmsg) < 0 ) {
+        return -1;
+    }
+
+    return $flag;
+}
+
+#
+# subnet,hostΥǡΥåȳǼ
+#
+# 
+#	arg0	subnetιԥǡ()
+#	arg1	subnetΥǡ(Ϣ:Ϥ)
+#	arg2	subnethostǡ(:Ϥ)
+#	arg4	ĥץ
+#	arg5	顼å
+#
+# ֤
+#	0	
+#	1	subnet顼orhost̵orhost顼
+#	2	顼(subnet̵)
+#
+
+sub get_subnet_host_data 
+{
+    my ($linenum, $data, $subnetdata, $allhostdata, $optionname, $errmsg) = @_;
+    my @allsubnetdata = @{$data};
+    my $status = 1;  #ơ        0: subnet
+                     #                  1: subnet
+                     #                  2: host
+    my @hostline;
+    my %hostdata;
+    my $checkline;
+    my $slinenum = $linenum;
+    my $hnum = 0;
+    my $ret;
+    my $starthostline;
+    my $checkflag = 0;    # ֤ѥե饰
+    my $hflag = 0;        # hostѤΥե饰
+    my $hnum = 0;         # hostθĿ
+    my $starthoststr;     # åʸ
+    my $endhoststr;       # åʸ
+
+    # åʸκ
+    $starthoststr = '^' . $STR_HOST . '\s+.*{$';
+    $endhoststr =  '^' . $STR_END . '$';
+
+    # ۥȡsubnetǡγǼ
+    foreach $checkline (@allsubnetdata) {
+
+        # Կ
+        $slinenum++;
+
+        # host {
+        if ($checkline =~ /$starthoststr/i) {
+            # ǽγ̤ξ
+            if ($status == 1) {
+                # hostϤޤԿ
+                $starthostline = $slinenum;
+            }
+            $status++;
+        }
+
+        # }
+        if ($checkline =~ /}$/i) {
+            # ơ򸺤餹
+            $status--;
+
+            # host}ɤ
+            if ($status == 1) {
+                # ۥȤޤãȤե饰ΩƤ
+                $hflag = 1;
+            } else {
+                # ۥȤĤ̤ǤʤСιԤ
+                next;
+            }
+        }
+
+        # hostǤСhostlineɵ
+        if ($status >= 2) {
+            push(@hostline, $checkline);
+            next;
+        }
+
+        # hostä
+        if ($hflag == 1) {
+
+            # hostǡγǼؿϤ
+            if (get_all_host_data($starthostline, $allhostdata, 
+                                  \@hostline, \$hnum, $errmsg) > 0) {
+
+                # 顼ǵäƤϡե饰򤿤Ƥ
+                $checkflag = 1;
+            }
+
+            # hostƤ󡦥ۥȽλΥե饰
+            undef(@hostline);
+            $hflag = 0;
+
+            # ιԤ
+            next;
+        }
+
+        # hostǤϤʤsubnetιԤξ
+        if ($status == 1) {
+            # ,ȤξФơιԤ
+            if ($checkline =~ /^$/ || $checkline =~ /^#/) {
+                next;
+            }
+
+            # ĥץΥåǼ
+            # ĥץꤵƤ
+            if ($optionname ne "") {
+
+                # ĥץåؿϤ
+                $ret = check_subnetline_ext($checkline, $slinenum, $subnetdata
+                                            , $optionname, $errmsg);
+                # ʣ顼ξ
+                if($ret == 2) {
+                    return 2;
+                 }
+
+                # ĥץ󤬤
+                if ($ret == 0) {
+                    # ιԤ
+                    next;
+                }
+            }
+
+            # subnetԥå,ǼؿϤ
+            $ret = check_subnetline($checkline, $slinenum, $subnetdata, 
+                                    $errmsg);
+            # ʣ顼ξ
+            if($ret == 2) {
+                return 2;
+
+            # 顼ξ
+            } elsif ($ret == 1) {
+                $checkflag = 1;
+            }
+            next;
+        }
+
+        # subnetν
+        if ($status == 0) {
+
+            # ,Ȥξ
+            if ($checkline =~ /^$/ || $checkline =~ /^#/) {
+                next;
+            }
+
+            # Ԥξ
+            $$errmsg[$L_DISP] .= $LANG{$ENV{$DISPLANG}}{'ERR_DHCPD_CONF_FORM'} 
+                                . "(" . $LANG{$ENV{$DISPLANG}}{'ERR_LINE_STR'}
+                                . $slinenum . ")\n";
+            $$errmsg[$L_LOG] .= $LANG{$LOGLANG}{'ERR_DHCPD_CONF_FORM'} 
+                               . "(" . $LANG{$LOGLANG}{'ERR_LINE_STR'} 
+                               . $slinenum . ")\n";
+            $checkflag = 1;
+        }
+    }
+
+    return $checkflag;
+}
+
 1;
