@@ -23,8 +23,8 @@
 # upload.cgiѥ饤֥
 #
 # $RCSfile: upload.pl,v $
-# $Revision: 1.10 $
-# $Date: 2009/01/05 07:09:03 $
+# $Revision: 1.11 $
+# $Date: 2013/01/16 10:05:56 $
 #
 ##########################################################
 
@@ -108,6 +108,7 @@ sub regist_cvs2host
     my %allmacdata;
     my %allhostdata;
     my %allipdata;
+    my %shareddata;
     my $ret;
 
     # CSVեɹ
@@ -116,8 +117,8 @@ sub regist_cvs2host
     }
 
     # CSVեνʣå
-    $ret = csvdata2checkdata($csvdata, \%data, \%allmacdata,
-                                    \%allhostdata, \%allipdata, $errmsg);
+    $ret = csvdata2checkdata($csvdata, \%data, \%allmacdata, \%allhostdata,
+                             \%allipdata, \%shareddata, $errmsg);
 
     # 顼
     if ($ret != 0) {
@@ -125,8 +126,8 @@ sub regist_cvs2host
     }
 
     # CSVեϿǡνʣå
-    $ret = make_rangedata(\%data, \%allmacdata, \%allhostdata,
-                                                \%allipdata, $csvnum, $errmsg);
+    $ret = make_rangedata(\%data, \%allmacdata, \%allhostdata, \%allipdata,
+                          \%shareddata, $csvnum, $errmsg);
     return $ret;
 }
 
@@ -135,13 +136,14 @@ sub regist_cvs2host
 #
 # 
 #       arg0    CSVեΰ
-#       arg1    ֥ͥå
-#       arg2    ۥ
-#       arg3    MACɥ쥹
-#       arg4    IPɥ쥹
-#       arg5    ֡Ȥε
-#       arg6    Կ
-#       arg7    顼å
+#       arg1    Shared-network
+#       arg2    ֥ͥå
+#       arg3    ۥ
+#       arg4    MACɥ쥹
+#       arg5    IPɥ쥹
+#       arg6    ֡Ȥε
+#       arg7    Կ
+#       arg8    顼å
 #
 # ֤
 #       0          
@@ -150,7 +152,7 @@ sub regist_cvs2host
 
 sub conv2checkdata
 {
-    my ($csvline, $subnet, $host, $mac, $ip, $boot, $cn, $errmsg) = @_;
+    my ($csvline, $shared, $subnet, $host, $mac, $ip, $boot, $cn, $errmsg) = @_;
     my @tmp;
     my @errmsg;
     my $ret;
@@ -182,8 +184,8 @@ sub conv2checkdata
 
     }
 
-    # ֥ͥåȥå
-    if (check_subnetdir($tmp[$csvsubnet]) == 1) {
+    # ֥ͥåȥåShared-networkμ
+    if (check_subnetdir($tmp[$csvsubnet], $shared) == 1) {
         $$errmsg[$L_DISP] .= $LANG{$ENV{$DISPLANG}}{'ERR_CSV_INV'};
         $$errmsg[$L_DISP] .= $LANG{$ENV{$DISPLANG}}{'ERR_SUBNET_NOFOUND'};
         $$errmsg[$L_DISP] .= "(" . $LANG{$ENV{$DISPLANG}}{'ERR_LINE_STR'} . $cn . ")";
@@ -242,8 +244,8 @@ sub conv2checkdata
         }
 
         # ϰϤȥȥȥå
-        $ret = check_ipformat($tmp[$csvsubnet], $tmp[$csvip],
-                                                $addmsg, $errmsg);
+        $ret = check_ipformat($$shared, $tmp[$csvsubnet], $tmp[$csvip],
+                              $addmsg, $errmsg);
 
         # ƥ२顼
         if ($ret == -1) {
@@ -308,6 +310,7 @@ sub check_boot
 #
 # 
 #       arg0    ֥ͥåȥǥ쥯ȥ
+#       arg1    Shared-network(Ϥ)
 #
 # ֤
 #       0          ¸ߤ
@@ -315,11 +318,50 @@ sub check_boot
 #
 
 sub check_subnetdir {
-    my ($subnet) = @_;
+    my ($subnet, $shared) = @_;
     my $subnetdirpath;
+    my @tmpdirlist;
+    my @dirlist;
+    my $filepath;
+    my $checksubnet;
+    my $line;
 
     $subnet =~ s/\//-/g;
-    $subnetdirpath = $WORKDIR . "/" . $subnet;
+    $$shared = "";
+
+    # Shared-networkĴ٤
+    # workǥ쥯ȥɤ߽Ф
+    if (read_dir($WORKDIR, \@tmpdirlist, $errmsg) < 0 ) {
+        return -1;
+    }
+
+    foreach $filepath (@tmpdirlist) {
+        # 켡ե(.lock)commonեФ
+        if ($filepath =~ /\/.lock$/ || $filepath =~ /\/common$/) {
+            next;
+        }
+
+        # Shared-network۲Υǥ쥯ȥɤ߽Ф
+        if (read_dir($filepath, \@dirlist, $errmsg) < 0 ) {
+            return -1;
+        }
+
+        foreach $line (@dirlist) {
+            $checksubnet = basename($line);
+
+            # ֥ͥåȤȰפShared-networkݻ
+            if ($checksubnet eq $subnet) {
+                $$shared = basename($filepath);
+                last;
+            }
+        }
+        # Ǥ˸ĤäƤ롼פȴ
+        if ($$shared ne "") {
+            last;
+        }
+    }
+
+    $subnetdirpath = $WORKDIR . "/" . $$shared . "/" . $subnet;
 
     # ʤ۾
     if ($subnet eq "") {
@@ -342,7 +384,8 @@ sub check_subnetdir {
 #       arg1    Ѵ̳Ǽ
 #       arg2    ٤ƤMACɥ쥹
 #       arg3    ٤ƤIPɥ쥹
-#       arg4    顼å
+#       arg4    ֥ͥåȤShared-networkɳդ
+#       arg5    顼å
 #
 # ֤
 #       0          
@@ -351,8 +394,8 @@ sub check_subnetdir {
 
 sub csvdata2checkdata
 {
-    my ($csvdata, $data, $allmacdata,
-                  $allhostdata, $allipdata, $errmsg) = @_;
+    my ($csvdata, $data, $allmacdata, $allhostdata, $allipdata, $shareddata, 
+        $errmsg) = @_;
     my $cn = 0;
     my $ret;
     my $convmac;
@@ -388,7 +431,8 @@ sub csvdata2checkdata
         } 
 
         # CSVǡηѴȥå
-        $ret = conv2checkdata($_, \$subnet, \$host, \$mac, \$ip, \$boot, $cn, $errmsg);
+        $ret = conv2checkdata($_, \$shared, \$subnet, \$host, \$mac, \$ip,
+                              \$boot, $cn, $errmsg);
 
         # ƥ२顼
         if ($ret == -1) {
@@ -430,6 +474,9 @@ sub csvdata2checkdata
         # ǡǼ
         $$data{$subnet}{$host} = sprintf("%s,%s,%s,%s", $host, $mac, $ip, $boot);
 
+        # Shared-networkǼ
+        $$shareddata{$subnet} = $shared;
+
         # MACɥ쥹ȹԿǼ
         $$allmacdata{"$subnet,$mac"} = $cn;
 
@@ -455,8 +502,9 @@ sub csvdata2checkdata
 #       arg1    MACɥ쥹
 #       arg2    ۥȥǡ
 #       arg3    IPǡ
-#       arg4    Կ
-#       arg5    顼å
+#       arg4    ֥ͥåȤShared-networkɳդǡ
+#       arg5    Կ
+#       arg6    顼å
 #
 # ֤
 #       0          
@@ -466,7 +514,8 @@ sub csvdata2checkdata
 
 sub make_rangedata
 {
-    my ($csvdata, $allmacdata, $allhostdata, $allipdata, $csvnum, $errmsg) = @_;
+    my ($csvdata, $allmacdata, $allhostdata, $allipdata, $shareddata, $csvnum,
+        $errmsg) = @_;
     my $subnetdir;
     my %hostlist;
     my %fixiplist;
@@ -487,12 +536,13 @@ sub make_rangedata
 
         # ʣå
         if (is_duplicate(\%hostlist, \%fixiplist, $csvdata, $allmacdata,
-                                     $allhostdata, $allipdata, $subnetdir, $errmsg) == 1) {
+                         $allhostdata, $allipdata, $subnetdir, $errmsg) == 1) {
             return 1;
         }
 
         # ֥ͥå˥ե˽
-        if (write_subnet_host($subnetdir, $subnet, $csvdata, $csvnum, $errmsg) == -1 ) {
+        if (write_subnet_host($subnetdir, $subnet, $csvdata, $csvnum,
+                              $shareddata, $errmsg) == -1 ) {
             return 1;
         }
     }
@@ -503,7 +553,7 @@ sub make_rangedata
         $subnetdir = $subnet;
         $subnetdir =~ s/\//\-/;
 
-        if (rename_subnet_host($subnetdir) == -1) {
+        if (rename_subnet_host($subnetdir, $subnet, $shareddata) == -1) {
             return -1;
         }
     }
@@ -516,6 +566,8 @@ sub make_rangedata
 #
 # 
 #       arg0    ֥ͥåȥǥ쥯ȥ
+#       arg1    ֥ͥå
+#       arg2    ֥ͥåȤShared-networkɳդǡ
 #
 # ֤
 #       0          
@@ -524,13 +576,13 @@ sub make_rangedata
 
 sub rename_subnet_host
 {
-    my ($subnetdir) = @_;
+    my ($subnetdir, $subnet, $shareddata) = @_;
     my $host;
     my $hosttmp;
 
     # ۥȥեΥѥ̾
-    $host = $WORKDIR . "/" . $subnetdir . "/host";
-    $hosttmp = $WORKDIR . "/" . $subnetdir . "/host.tmp";
+    $host = $WORKDIR . "/" . $$shareddata{$subnet} . "/" . $subnetdir . "/host";
+    $hosttmp = $WORKDIR . "/" . $$shareddata{$subnet} . "/" . $subnetdir . "/host.tmp";
 
     if (rename($hosttmp, $host) == 0) {
         return -1;
@@ -547,7 +599,8 @@ sub rename_subnet_host
 #       arg1    ֥ͥå
 #       arg2    CSVѴѤߥǡ
 #       arg3    Կ
-#       arg4    顼å
+#       arg4    ֥ͥåȤShared-networkɳդǡ
+#       arg5    顼å
 #
 # ֤
 #       0          
@@ -557,15 +610,15 @@ sub rename_subnet_host
 
 sub write_subnet_host
 {
-    my ($subnetdir, $subnet, $csvdata, $csvnum, $errmsg) = @_;
+    my ($subnetdir, $subnet, $csvdata, $csvnum, $shareddata, $errmsg) = @_;
     my $data = "";
     my $host;
     my $hosttmp;
     my $ret;
 
     # ۥȥեΥѥ̾
-    $host = $WORKDIR . "/" . $subnetdir . "/host";
-    $hosttmp = $WORKDIR . "/" . $subnetdir . "/host.tmp";
+    $host = $WORKDIR . "/" . $$shareddata{$subnet} . "/" . $subnetdir . "/host";
+    $hosttmp = $WORKDIR . "/" . $$shareddata{$subnet} . "/" . $subnetdir . "/host.tmp";
 
     # ǡ
     foreach (sort keys %{$$csvdata{$subnet}}) {
